@@ -1,11 +1,11 @@
-;;; concurrent.el --- Concurrent utility functions for emacs lisp
+;;; concurrent.el --- Concurrent utility functions for emacs lisp  -*- lexical-binding: t; -*-
 
 ;; Copyright (C) 2010-2016  SAKURAI Masashi
 
 ;; Author: SAKURAI Masashi <m.sakurai at kiwanami.net>
-;; Version: 0.4.0
+;; Version: 0.5.0
 ;; Keywords: deferred, async, concurrent
-;; Package-Requires: ((deferred "0.4.0"))
+;; Package-Requires: ((emacs "24.3") (deferred "0.5.0"))
 ;; URL: https://github.com/kiwanami/emacs-deferred/blob/master/README-concurrent.markdown
 
 ;; This program is free software; you can redistribute it and/or modify
@@ -32,7 +32,7 @@
 ;; - Dataflow
 ;; - Signal/Channel
 
-(require 'cl)
+(require 'cl-lib)
 
 (require 'deferred)
 
@@ -55,15 +55,15 @@
 (defun cc:generator-replace-yield (tree)
   "[internal] Replace `yield' symbols to calling a function in TREE."
   (let (ret)
-    (loop for i in tree
-          do (cond
-              ((eq i 'yield)
-               (push 'funcall ret)
-               (push i ret))
-              ((listp i)
-               (push (cc:generator-replace-yield i) ret))
-              (t
-               (push i ret))))
+    (cl-loop for i in tree
+             do (cond
+                 ((eq i 'yield)
+                  (push 'funcall ret)
+                  (push i ret))
+                 ((listp i)
+                  (push (cc:generator-replace-yield i) ret))
+                 (t
+                  (push i ret))))
     (nreverse ret)))
 
 (defun cc:generator-line (chain line)
@@ -93,18 +93,17 @@ asynchronously."
 (defmacro cc:generator (callback &rest body)
   "Create a generator object. If BODY has `yield' symbols, it
 means calling callback function CALLBACK."
-  (let ((chain (gensym))
-        (cc (gensym))
-        (waiter (gensym)))
-    `(lexical-let*
-         (,chain
-          (,cc ,callback)
-          (,waiter (deferred:new))
-          (yield (lambda (x) (funcall ,cc x) ,waiter)))
+  (let ((chain (cl-gensym))
+        (cc (cl-gensym))
+        (waiter (cl-gensym)))
+    `(let* (,chain
+            (,cc ,callback)
+            (,waiter (deferred:new))
+            (yield (lambda (x) (funcall ,cc x) ,waiter)))
        (setq ,chain ,waiter)
-       ,@(loop for i in body
-               collect
-               (cc:generator-line chain i))
+       ,@(cl-loop for i in body
+                  collect
+                  (cc:generator-line chain i))
        (lambda () (deferred:callback ,waiter)))))
 
 
@@ -124,7 +123,7 @@ CHAIN is the previous deferred task."
    ((eq 'while (car line))
     (let ((condition (cadr line))
           (body (cddr line))
-          (retsym (gensym)))
+          (retsym (cl-gensym)))
     `(setq ,chain
       (deferred:nextc ,chain
         (deferred:lambda (x)
@@ -142,15 +141,14 @@ CHAIN is the previous deferred task."
 
 (defmacro cc:thread (wait-time-msec &rest body)
   "Return a thread object."
-  (let ((chain (gensym))
-        (dstart (gensym)))
-    `(lexical-let*
-         (,chain
-          (,dstart (deferred:new)))
+  (let ((chain (cl-gensym))
+        (dstart (cl-gensym)))
+    `(let* (,chain
+            (,dstart (deferred:new)))
        (setq ,chain ,dstart)
-       ,@(loop for i in body
-               collect
-               (cc:thread-line wait-time-msec chain i))
+       ,@(cl-loop for i in body
+                  collect
+                  (cc:thread-line wait-time-msec chain i))
        (deferred:callback ,dstart))))
 (put 'cc:thread 'lisp-indent-function 1)
 
@@ -159,7 +157,7 @@ CHAIN is the previous deferred task."
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Semaphore
 
-(defstruct cc:semaphore max-permits permits waiting-deferreds)
+(cl-defstruct cc:semaphore max-permits permits waiting-deferreds)
 
 (defun cc:semaphore-create(permits-num)
   "Return a semaphore object with PERMITS-NUM permissions."
@@ -173,7 +171,7 @@ permissions, the subsequent deferred task is blocked. After the
 permission is returned, the task is executed."
   (cond
    ((< 0 (cc:semaphore-permits semaphore))
-    (decf (cc:semaphore-permits semaphore))
+    (cl-decf (cc:semaphore-permits semaphore))
     (deferred:succeed))
    (t
     (let ((d (deferred:new)))
@@ -196,18 +194,17 @@ permission is returned, the task is executed."
               (nbutlast waiting-deferreds))
         (deferred:callback-post d)))
      (t
-      (incf (cc:semaphore-permits semaphore)))))
+      (cl-incf (cc:semaphore-permits semaphore)))))
   semaphore)
 
 (defun cc:semaphore-with (semaphore body-func &optional error-func)
   "Execute the task BODY-FUNC asynchronously with the semaphore block."
-  (lexical-let ((semaphore semaphore))
-    (deferred:try
-      (deferred:nextc (cc:semaphore-acquire semaphore) body-func)
-      :catch
-      error-func
-      :finally
-      (lambda (_x) (cc:semaphore-release semaphore)))))
+  (deferred:try
+    (deferred:nextc (cc:semaphore-acquire semaphore) body-func)
+    :catch
+    error-func
+    :finally
+    (lambda (_x) (cc:semaphore-release semaphore))))
 (put 'cc:semaphore-with 'lisp-indent-function 1)
 
 (defun cc:semaphore-release-all (semaphore)
@@ -240,17 +237,16 @@ This function is used for the interruption cases."
 NAME is a channel name for debug.
 PARENT-CHANNEL is an upstream channel. The observers of this channel can receive the upstream signals.
 In the case of using the function `cc:signal-send', the observers of the upstream channel can not receive the signals of this channel. The function `cc:signal-send-global' can send a signal to the upstream channels from the downstream channels."
-  (lexical-let
-      ((ch (cons
-            (or name (format "signal%s" (deferred:uid))) ; name for debug
-            (cons
-             parent-channel ; parent-channel
-             nil)))) ; observers
+  (let ((ch (cons
+             (or name (format "signal%s" (deferred:uid))) ; name for debug
+             (cons
+              parent-channel ; parent-channel
+              nil)))) ; observers
     (when parent-channel
       (cc:signal-connect
        parent-channel
        t (lambda (event)
-           (destructuring-bind
+           (cl-destructuring-bind
                (event-name event-args) event
              (apply 'cc:signal-send
                     ch event-name event-args)))))
@@ -285,11 +281,11 @@ tasks to the returned deferred object."
   "Send a signal to CHANNEL. If ARGS values are given, observers can get the values by following code: (lambda (event) (destructuring-bind (event-sym (args)) event ... )). "
   (let ((observers (cc:signal-observers channel))
         (event (list event-sym args)))
-    (loop for i in observers
-          for name = (car i)
-          for d = (cdr i)
-          if (or (eq event-sym name) (eq t name))
-          do (deferred:callback-post d event))))
+    (cl-loop for i in observers
+             for name = (car i)
+             for d = (cdr i)
+             if (or (eq event-sym name) (eq t name))
+             do (deferred:callback-post d event))))
 
 (defun cc:signal-send-global (channel event-sym &rest args)
   "Send a signal to the most upstream channel. "
@@ -304,12 +300,12 @@ the removed deferred object. "
   (let ((observers (cc:signal-observers channel)) deleted)
     (setf
      (cc:signal-observers channel) ; place
-     (loop for i in observers
-           for d = (cdr i)
-           unless (eq d deferred)
-           collect i
-           else
-           do (push i deleted)))
+     (cl-loop for i in observers
+              for d = (cdr i)
+              unless (eq d deferred)
+              collect i
+              else
+              do (push i deleted)))
     deleted))
 
 (defun cc:signal-disconnect-all (channel)
@@ -325,7 +321,7 @@ the removed deferred object. "
 ;; Dataflow
 
 ;; Dataflow variable entry
-(defstruct cc:dataflow key (value 'cc:dataflow-undefine) deferred-list)
+(cl-defstruct cc:dataflow key (value 'cc:dataflow-undefine) deferred-list)
 
 (defun cc:dataflow-undefine-p (obj)
   "[internal] If the variable entry is not bound, return `t'."
@@ -341,11 +337,11 @@ the removed deferred object. "
 
 (defmacro cc:dataflow-channel (df)
   "[internal] Return the channel object."
-  `(caddr ,df))
+  `(cl-caddr ,df))
 
 (defmacro cc:dataflow-list (df)
   "[internal] Return the list of deferred object which are waiting for value binding."
-  `(cdddr ,df))
+  `(cl-cdddr ,df))
 
 (defun cc:dataflow-environment (&optional parent-env test-func channel)
   "Create a dataflow environment.
@@ -371,28 +367,27 @@ CHANNEL is a channel object that sends signals of variable events. Observers can
 
 (defun cc:dataflow-init-connect (df)
   "[internal] Initialize the channel object."
-  (lexical-let ((df df))
-    (cc:dataflow-connect
-     df 'set
-     (lambda (args)
-       (destructuring-bind (_event (key)) args
-         (let* ((obj (cc:dataflow-get-object-for-value df key))
-                (value (and obj (cc:dataflow-value obj))))
-           (when obj
-             (loop for i in (cc:aif (cc:dataflow-get-object-for-deferreds df key)
-                                (cc:dataflow-deferred-list it) nil)
-                   do (deferred:callback-post i value))
-             (setf (cc:dataflow-deferred-list obj) nil))))))))
+  (cc:dataflow-connect
+   df 'set
+   (lambda (args)
+     (cl-destructuring-bind (_event (key)) args
+       (let* ((obj (cc:dataflow-get-object-for-value df key))
+              (value (and obj (cc:dataflow-value obj))))
+         (when obj
+           (cl-loop for i in (cc:aif (cc:dataflow-get-object-for-deferreds df key)
+                                     (cc:dataflow-deferred-list it) nil)
+                    do (deferred:callback-post i value))
+           (setf (cc:dataflow-deferred-list obj) nil)))))))
 
 (defun cc:dataflow-get-object-for-value (df key)
   "[internal] Return an entry object that is indicated by KEY.
 If the environment DF doesn't have the entry and the parent one has the entry, this function returns the entry of the parent environment. This function doesn't affect the waiting list."
   (or
-   (loop for i in (cc:dataflow-list df)
-         with test = (cc:dataflow-test df)
-         if (and (funcall test key (cc:dataflow-key i))
-                 (not (cc:dataflow-undefine-p i)))
-         return i)
+   (cl-loop for i in (cc:dataflow-list df)
+            with test = (cc:dataflow-test df)
+            if (and (funcall test key (cc:dataflow-key i))
+                    (not (cc:dataflow-undefine-p i)))
+            return i)
    (deferred:aand
      (cc:dataflow-parent-environment df)
      (cc:dataflow-get-object-for-value it key))))
@@ -400,10 +395,10 @@ If the environment DF doesn't have the entry and the parent one has the entry, t
 (defun cc:dataflow-get-object-for-deferreds (df key)
   "[internal] Return a list of the deferred objects those are waiting for value binding.
 This function doesn't affect the waiting list and doesn't refer the parent environment."
-  (loop for i in (cc:dataflow-list df)
-        with test = (cc:dataflow-test df)
-        if (funcall test key (cc:dataflow-key i))
-        return i))
+  (cl-loop for i in (cc:dataflow-list df)
+           with test = (cc:dataflow-test df)
+           if (funcall test key (cc:dataflow-key i))
+           return i))
 
 (defun cc:dataflow-connect (df event-sym &optional callback)
   "Append an observer for EVENT-SYM of the channel of DF and return a deferred object.
@@ -464,18 +459,18 @@ VALUE can be nil as a value."
 This function does nothing for the waiting deferred objects."
   (cc:dataflow-signal df 'clear key)
   (setf (cc:dataflow-list df)
-        (loop for i in (cc:dataflow-list df)
-              with test = (cc:dataflow-test df)
-              unless (funcall test key (cc:dataflow-key i))
-              collect i)))
+        (cl-loop for i in (cc:dataflow-list df)
+                 with test = (cc:dataflow-test df)
+                 unless (funcall test key (cc:dataflow-key i))
+                 collect i)))
 
 (defun cc:dataflow-get-avalable-pairs (df)
   "Return an available key-value alist in the environment DF and the parent ones."
   (append
-   (loop for i in (cc:dataflow-list df)
-         for key = (cc:dataflow-key i)
-         for val = (cc:dataflow-value i)
-         unless (cc:dataflow-undefine-p i) collect (cons key val))
+   (cl-loop for i in (cc:dataflow-list df)
+            for key = (cc:dataflow-key i)
+            for val = (cc:dataflow-value i)
+            unless (cc:dataflow-undefine-p i) collect (cons key val))
    (deferred:aand
      (cc:dataflow-parent-environment df)
      (cc:dataflow-get-avalable-pairs it))))
@@ -483,9 +478,9 @@ This function does nothing for the waiting deferred objects."
 (defun cc:dataflow-get-waiting-keys (df)
   "Return a list of keys which have waiting deferred objects in the environment DF and the parent ones."
   (append
-   (loop for i in (cc:dataflow-list df)
-         for key = (cc:dataflow-key i)
-         if (cc:dataflow-undefine-p i) collect key)
+   (cl-loop for i in (cc:dataflow-list df)
+            for key = (cc:dataflow-key i)
+            if (cc:dataflow-undefine-p i) collect key)
    (deferred:aand
      (cc:dataflow-parent-environment df)
      (cc:dataflow-get-waiting-keys it))))
