@@ -3,9 +3,9 @@
 ;; Copyright (C) 2010-2016 SAKURAI Masashi
 
 ;; Author: SAKURAI Masashi <m.sakurai at kiwanami.net>
-;; Version: 0.5.0
+;; Version: 0.5.1
 ;; Keywords: deferred, async
-;; Package-Requires: ((emacs "24.3"))
+;; Package-Requires: ((emacs "24.4"))
 ;; URL: https://github.com/kiwanami/emacs-deferred
 
 ;; This program is free software; you can redistribute it and/or modify
@@ -67,11 +67,12 @@
 ;; deferred.el.
 
 (require 'cl-lib)
+(require 'subr-x)
 
 (declare-function pp-display-expression 'pp)
 
 (defvar deferred:version nil "deferred.el version")
-(setq deferred:version "0.4.0")
+(setq deferred:version "0.5.0")
 
 ;;; Code:
 
@@ -743,32 +744,32 @@ deferred task and return the TIMEOUT-FORM."
   "A deferred wrapper of `start-process'. Return a deferred
 object. The process name and buffer name of the argument of the
 `start-process' are generated by this function automatically.
-The next deferred object receives stdout string from the command
-process."
+The next deferred object receives stdout and stderr string from
+the command process."
   (deferred:process-gen 'start-process command args))
 
 (defun deferred:process-shell (command &rest args)
   "A deferred wrapper of `start-process-shell-command'. Return a deferred
 object. The process name and buffer name of the argument of the
 `start-process-shell-command' are generated by this function automatically.
-The next deferred object receives stdout string from the command
-process."
+The next deferred object receives stdout and stderr string from
+the command process."
   (deferred:process-gen 'start-process-shell-command command args))
 
 (defun deferred:process-buffer (command &rest args)
   "A deferred wrapper of `start-process'. Return a deferred
 object. The process name and buffer name of the argument of the
 `start-process' are generated by this function automatically.
-The next deferred object receives stdout buffer from the command
-process."
+The next deferred object receives stdout and stderr buffer from
+the command process."
   (deferred:process-buffer-gen 'start-process command args))
 
 (defun deferred:process-shell-buffer (command &rest args)
   "A deferred wrapper of `start-process-shell-command'. Return a deferred
 object. The process name and buffer name of the argument of the
 `start-process-shell-command' are generated by this function automatically.
-The next deferred object receives stdout buffer from the command
-process."
+The next deferred object receives stdout and stderr buffer from
+the command process."
   (deferred:process-buffer-gen 'start-process-shell-command command args))
 
 (defun deferred:process-gen (f command args)
@@ -807,25 +808,28 @@ process."
                         (apply f proc-name buf-name command args)))
                 (set-process-sentinel
                  proc
-                 (lambda (_proc event)
-                   (cond
-                    ((string-match "exited abnormally" event)
-                     (let ((msg (if (buffer-live-p proc-buf)
-                                    (format "Process [%s] exited abnormally : %s"
-                                            command
-                                            (with-current-buffer proc-buf (buffer-string)))
-                                  (concat "Process exited abnormally: " proc-name))))
-                       (kill-buffer proc-buf)
-                       (deferred:post-task nd 'ng msg)))
-                    ((equal event "finished\n")
-                     (deferred:post-task nd 'ok proc-buf)))))
-                (setf (deferred-cancel nd)
-                      (lambda (x) (deferred:default-cancel x)
-                        (when proc
-                          (kill-process proc)
-                          (kill-buffer proc-buf)))))
-            (error (deferred:post-task nd 'ng err)))
-          nil))
+                 (lambda (proc event)
+		   (unless (process-live-p proc)
+		     (if (zerop (process-exit-status proc))
+			 (deferred:post-task nd 'ok proc-buf)
+		       (let ((msg (format "Deferred process exited abnormally:\n  command: %s\n  exit status: %s %s\n  event: %s\n  buffer contents: %S"
+					  command
+					  (process-status proc)
+					  (process-exit-status proc)
+					  (string-trim-right event)
+					  (if (buffer-live-p proc-buf)
+					      (with-current-buffer proc-buf
+						(buffer-string))
+					    "(unavailable)"))))
+			 (kill-buffer proc-buf)
+			 (deferred:post-task nd 'ng msg))))))
+		(setf (deferred-cancel nd)
+		      (lambda (x) (deferred:default-cancel x)
+			(when proc
+			  (kill-process proc)
+			  (kill-buffer proc-buf)))))
+	    (error (deferred:post-task nd 'ng err)))
+	  nil))
       nd)))
 
 (defmacro deferred:processc (d command &rest args)
