@@ -1,4 +1,4 @@
-;;; test code for concurrent.el
+;;; test code for concurrent.el  -*- lexical-binding: t; -*-
 
 ;; Copyright (C) 2010  SAKURAI Masashi
 ;; Author: SAKURAI Masashi <m.sakurai at kiwanami.net>
@@ -24,7 +24,7 @@
             (:send-report nil)
             (:report-file "/tmp/undercover-report.json"))
 (require 'concurrent)
-(require 'cl)
+(require 'cl-lib)
 (require 'pp)
 (require 'ert)
 
@@ -35,8 +35,8 @@
 ;; generator
 
 (defun cc:fib-gen (callback)
-  (lexical-let ((a1 0) (a2 1)
-                (callback callback))
+  (let ((a1 0) (a2 1)
+        (callback callback))
         (cc:generator
          callback
          (yield a1)
@@ -48,20 +48,19 @@
              (yield next))))))
 
 (defun cc:test-fib-gen ()
-  (lexical-let*
-      ((count 0)
-       (dfinish (deferred:new))
-       gen
-       (cc (lambda (x)
-             (cond
-              ((= count 10)
-               (deferred:callback
-                 dfinish
-                 (if (= x 55) t
-                   (format "Fib 10 = 55 -> %s" x))))
-              (t
-               (incf count)
-               (deferred:call gen))))))
+  (let* ((count 0)
+         (dfinish (deferred:new))
+         gen
+         (cc (lambda (x)
+               (cond
+                ((= count 10)
+                 (deferred:callback
+                   dfinish
+                   (if (= x 55) t
+                     (format "Fib 10 = 55 -> %s" x))))
+                (t
+                 (cl-incf count)
+                 (deferred:call gen))))))
     (setq gen (cc:fib-gen cc))
     (deferred:call gen)
     dfinish))
@@ -71,15 +70,14 @@
 ;; thread
 
 (defun cc:test-thread ()
-  (lexical-let
-      ((dfinish (deferred:new))
-       (result nil) (start-time (float-time))
-       (count 0) (end 20))
+  (let ((dfinish (deferred:new))
+        (result nil) (start-time (float-time))
+        (count 0) (end 20))
     (push 1 result)
     (cc:thread
      60
      (push 2 result)
-     (while (> end (incf count))
+     (while (> end (cl-incf count))
        (when (= 0 (% count 10))
          (push count result)))
      (push 99 result)
@@ -95,87 +93,85 @@
 ;; semaphore
 
 (defun cc:test-semaphore1 ()
-  (lexical-let*
-      ((result nil)
-       (dfinish (deferred:new
-                  (lambda (x)
-                    (setq result (reverse result))
-                    (or (equal '(1 2 5 6 (size . 1) 3 7 8 canceled (size . 0)) result)
-                        result))))
-       (smp (cc:semaphore-create 1)))
+  (let* ((result nil)
+         (dfinish (deferred:new
+                    (lambda (_)
+                      (setq result (reverse result))
+                      (or (equal '(1 2 5 6 (size . 1) 3 7 8 canceled (size . 0)) result)
+                          result))))
+         (smp (cc:semaphore-create 1)))
 
     (push 1 result)
 
     (deferred:nextc (cc:semaphore-acquire smp)
-      (lambda(x) (push 2 result)))
+      (lambda(_) (push 2 result)))
     (deferred:nextc (cc:semaphore-acquire smp)
-      (lambda(x) (push 3 result)))
+      (lambda(_) (push 3 result)))
     (deferred:nextc (cc:semaphore-acquire smp)
       (lambda(x) (push x result)))
 
     (deferred:$
       (deferred:next
-        (lambda (x)
+        (lambda (_)
           (push 5 result)
           (cc:semaphore-release smp)
           (push 6 result)))
       (deferred:nextc it
-        (lambda (x)
+        (lambda (_)
           (push (cons 'size (length (cc:semaphore-waiting-deferreds smp))) result)))
       (deferred:nextc it
-        (lambda (x)
+        (lambda (_)
           (push 7 result)
-          (loop for i in (cc:semaphore-release-all smp)
-                do (deferred:callback i 'canceled))
+          (cl-loop for i in (cc:semaphore-release-all smp)
+                   do (deferred:callback i 'canceled))
           (push 8 result)))
       (deferred:nextc it
-        (lambda (x)
+        (lambda (_)
           (push (cons 'size (length (cc:semaphore-waiting-deferreds smp))) result)))
       (deferred:nextc it
-        (lambda (x) (deferred:callback dfinish))))
+        (lambda (_) (deferred:callback dfinish))))
 
     dfinish))
 
 ;; (cc:debug (cc:test-semaphore1) "Semaphore1 : %s" x)
 
 (defun cc:test-semaphore2 ()
-  (lexical-let*
-      ((result nil)
-       (dfinish (deferred:new
-                  (lambda (x)
-                    (setq result (reverse result))
-                    (or (equal '(0 a b c d e f g) result)
-                        result))))
-       (smp (cc:semaphore-create 1)))
+  (let* ((result nil)
+         (dfinish (deferred:new
+                    (lambda (_)
+                      (setq result (reverse result))
+                      (or (equal '(0 a b c d e f g) result)
+                          result))))
+         (smp (cc:semaphore-create 1)))
 
     (push 0 result)
 
     (cc:semaphore-with
-     smp (lambda (x)
+     smp (lambda (_)
            (deferred:nextc (cc:semaphore-acquire smp)
-             (lambda (x)
+             (lambda (_)
                (push 'c result)
                (cc:semaphore-release smp)))
            (push 'a result)
            (deferred:nextc
              (deferred:wait 100)
-             (lambda (x) (push 'b result)))))
+             (lambda (_) (push 'b result)))))
 
     (cc:semaphore-with
-     smp (lambda (x)
+     smp (lambda (_)
            (deferred:nextc (cc:semaphore-acquire smp)
-             (lambda (x)
+             (lambda (_)
                (push 'g result)
                (cc:semaphore-release smp)
                (deferred:callback dfinish)))
              (push 'd result)
              (deferred:nextc
                (deferred:wait 100)
-               (lambda (x)
+               (lambda (_)
                  (push 'e result)
                  (error "SMP CC ERR"))))
      (lambda (e)
-       (destructuring-bind (sym msg) e
+       (cl-destructuring-bind (sym msg) e
          (when (and (eq 'error sym) (equal "SMP CC ERR" msg))
            (push 'f result)))))
 
@@ -186,14 +182,13 @@
 ;; Dataflow
 
 (defun cc:test-dataflow-simple1 ()
-  (lexical-let*
-      ((result '(1))
-       (dfinish (deferred:new
-                  (lambda (x)
-                    (setq result (reverse result))
-                    (or (equal '(1 (2 . nil) 4 5 (3 . 256) (6 . 256) (7 . nil)) result)
-                        result))))
-       (dfenv (cc:dataflow-environment)))
+  (let* ((result '(1))
+         (dfinish (deferred:new
+                    (lambda (_)
+                      (setq result (reverse result))
+                      (or (equal '(1 (2 . nil) 4 5 (3 . 256) (6 . 256) (7 . nil)) result)
+                          result))))
+         (dfenv (cc:dataflow-environment)))
 
     (push (cons 2 (cc:dataflow-get-sync dfenv "aaa")) result)
 
@@ -205,17 +200,17 @@
             (lambda (x) (push (cons 3 x) result))))
         (deferred:$
           (deferred:next
-            (lambda (x)
+            (lambda (_)
               (push 4 result)
               (cc:dataflow-set dfenv "abc" 256)
               (push 5 result)))))
       (deferred:nextc it
-        (lambda (x)
+        (lambda (_)
           (push (cons 6 (cc:dataflow-get-sync dfenv "abc")) result)
           (cc:dataflow-clear dfenv "abc")
           (push (cons 7 (cc:dataflow-get-sync dfenv "abc")) result)))
       (deferred:nextc it
-        (lambda (x)
+        (lambda (_)
           (deferred:callback dfinish))))
 
     dfinish))
@@ -223,20 +218,19 @@
 ;; (cc:debug (cc:test-dataflow-simple1) "Dataflow1 : %s" x)
 
 (defun cc:test-dataflow-simple2 ()
-  (lexical-let*
-      ((result nil)
-       (dfinish (deferred:new
-                  (lambda (x)
-                    (or (equal '("a.jpg:300 OK jpeg") result)
-                        result))))
-       (dfenv (cc:dataflow-environment)))
+  (let* ((result nil)
+         (dfinish (deferred:new
+                    (lambda (_)
+                      (or (equal '("a.jpg:300 OK jpeg") result)
+                          result))))
+         (dfenv (cc:dataflow-environment)))
 
     (deferred:$
       (cc:dataflow-get dfenv '("http://example.com/a.jpg" 300))
       (deferred:nextc it
         (lambda (x) (push (format "a.jpg:300 OK %s" x) result)))
       (deferred:nextc it
-        (lambda (x)
+        (lambda (_)
           (deferred:callback dfinish))))
 
     (cc:dataflow-set dfenv '("http://example.com/a.jpg" 300) 'jpeg)
@@ -246,13 +240,12 @@
 ;; (cc:debug (cc:test-dataflow-simple2) "Dataflow2 : %s" x)
 
 (defun cc:test-dataflow-simple3 ()
-  (lexical-let*
-      ((result nil)
-       (dfinish (deferred:new
-                  (lambda (x)
-                    (or (equal '(">> 384") result)
-                        result))))
-       (dfenv (cc:dataflow-environment)))
+  (let* ((result nil)
+         (dfinish (deferred:new
+                    (lambda (_)
+                      (or (equal '(">> 384") result)
+                          result))))
+         (dfenv (cc:dataflow-environment)))
 
     (deferred:$
       (deferred:parallel
@@ -264,11 +257,11 @@
       (deferred:nextc it
         (lambda (x) (push (format ">> %s" x) result)))
       (deferred:nextc it
-        (lambda (x)
+        (lambda (_)
           (deferred:callback dfinish))))
 
     (deferred:nextc (deferred:wait 0.2)
-      (lambda (x)
+      (lambda (_)
         (cc:dataflow-set dfenv "def" 128)
         (cc:dataflow-set dfenv "abc" 256)
         (cc:dataflow-set dfenv "aaa" 512)
@@ -279,13 +272,12 @@
 ;; (cc:debug (cc:test-dataflow-simple3) "Dataflow3 : %s" x)
 
 (defun cc:test-dataflow-simple4 ()
-  (lexical-let*
-      ((result nil)
-       (dfinish (deferred:new
-                  (lambda (x)
-                    (or (equal '(">> 3") result)
-                        result))))
-       (dfenv (cc:dataflow-environment)))
+  (let* ((result nil)
+         (dfinish (deferred:new
+                    (lambda (_)
+                      (or (equal '(">> 3") result)
+                          result))))
+         (dfenv (cc:dataflow-environment)))
 
     (deferred:$
       (deferred:parallel
@@ -298,11 +290,11 @@
       (deferred:nextc it
         (lambda (x) (push (format ">> %s" x) result)))
       (deferred:nextc it
-        (lambda (x)
+        (lambda (_)
           (deferred:callback dfinish))))
 
     (deferred:nextc (deferred:wait 0.2)
-      (lambda (x)
+      (lambda (_)
         (cc:dataflow-set dfenv "abc" 1)
         ))
 
@@ -311,31 +303,31 @@
 ;; (cc:debug (cc:test-dataflow-simple4) "Dataflow4 : %s" x)
 
 (defun cc:test-dataflow-signal ()
-  (lexical-let*
-      ((result '(1))
-       (dfinish (deferred:new
-                  (lambda (x)
-                    (setq result (reverse result))
-                    (or (equal
-                         '(1
-                           (2 . nil)
-                           (get-first ("abc"))
-                           (get-waiting ("abc"))
-                           4 5
-                           (set ("abc"))
-                           (3 . 256)
-                           6 7
-                           (get ("abc"))
-                           (8 . 256)
-                           (9 . nil)
-                           (clear ("abc"))
-                           (clear-all (nil))
-                           ) result)
-                        result))))
-       (dfenv (cc:dataflow-environment)))
+  (let* ((result '(1))
+         (dfinish (deferred:new
+                    (lambda (_)
+                      (setq result (reverse result))
+                      (or (equal
+                           '(1
+                             (2 . nil)
+                             (get-first ("abc"))
+                             (get-waiting ("abc"))
+                             4 5
+                             (set ("abc"))
+                             (3 . 256)
+                             6 7
+                             (get ("abc"))
+                             (8 . 256)
+                             (9 . nil)
+                             (clear ("abc"))
+                             (clear-all (nil))
+                             )
+                           result)
+                          result))))
+         (dfenv (cc:dataflow-environment)))
 
-    (loop for i in '(get get-first get-waiting set clear clear-all)
-          do (cc:dataflow-connect dfenv i (lambda (ev) (push ev result))))
+    (cl-loop for i in '(get get-first get-waiting set clear clear-all)
+             do (cc:dataflow-connect dfenv i (lambda (ev) (push ev result))))
 
     (push (cons 2 (cc:dataflow-get-sync dfenv "aaa")) result)
 
@@ -347,25 +339,25 @@
             (lambda (x) (push (cons 3 x) result))))
         (deferred:$
           (deferred:next
-            (lambda (x)
+            (lambda (_)
               (push 4 result)
               (cc:dataflow-set dfenv "abc" 256)
               (push 5 result)))))
       (deferred:nextc it
-        (lambda (x)
+        (lambda (_)
           (push 6 result)
           (cc:dataflow-get dfenv "abc")
           (push 7 result)))
       (deferred:nextc it
-        (lambda (x)
+        (lambda (_)
           (push (cons 8 (cc:dataflow-get-sync dfenv "abc")) result)
           (cc:dataflow-clear dfenv "abc")
           (push (cons 9 (cc:dataflow-get-sync dfenv "abc")) result)))
       (deferred:nextc it
-        (lambda (x)
+        (lambda (_)
           (cc:dataflow-clear-all dfenv)))
       (deferred:nextc it
-        (lambda (x)
+        (lambda (_)
           (deferred:callback dfinish))))
 
     dfinish))
@@ -374,23 +366,23 @@
 
 
 (defun cc:test-dataflow-parent1 ()
-  (lexical-let*
-      ((result '(1))
-       (dfinish (deferred:new
-                  (lambda (x)
-                    (setq result (reverse result))
-                    (or (equal
-                         '(1
-                           (available-parent . (("abc" . 128)))
-                           (available-child . (("abc" . 128)))
-                           (waiting-parent . nil)
-                           (waiting-child . ("aaa"))
-                           (get-sync . 256)
-                           (get . 256)
-                           ) result)
-                        result))))
-       (dfenv-parent (cc:dataflow-environment))
-       (dfenv (cc:dataflow-environment dfenv-parent)))
+  (let* ((result '(1))
+         (dfinish (deferred:new
+                    (lambda (_)
+                      (setq result (reverse result))
+                      (or (equal
+                           '(1
+                             (available-parent . (("abc" . 128)))
+                             (available-child . (("abc" . 128)))
+                             (waiting-parent . nil)
+                             (waiting-child . ("aaa"))
+                             (get-sync . 256)
+                             (get . 256)
+                             )
+                           result)
+                          result))))
+         (dfenv-parent (cc:dataflow-environment))
+         (dfenv (cc:dataflow-environment dfenv-parent)))
 
     (cc:dataflow-set dfenv-parent "abc" 128)
 
@@ -402,33 +394,32 @@
             (lambda (x) (push (cons 'get x) result))))
         (deferred:$
           (deferred:next
-            (lambda (x)
+            (lambda (_)
               (push (cons 'available-parent (cc:dataflow-get-avalable-pairs dfenv-parent)) result)
               (push (cons 'available-child  (cc:dataflow-get-avalable-pairs dfenv)) result)
               (push (cons 'waiting-parent (cc:dataflow-get-waiting-keys dfenv-parent)) result)
               (push (cons 'waiting-child  (cc:dataflow-get-waiting-keys dfenv)) result)))
           (deferred:next
-            (lambda (x)
+            (lambda (_)
               (cc:dataflow-set dfenv-parent "aaa" 256)
               (push (cons 'get-sync (cc:dataflow-get-sync dfenv "aaa")) result)))))
       (deferred:nextc it
-        (lambda (x) (deferred:callback dfinish))))
+        (lambda (_) (deferred:callback dfinish))))
 
     dfinish))
 
 ;; (cc:debug (cc:test-dataflow-parent1) "Dataflow Parent1 : %s" x)
 
 (defun cc:test-dataflow-parent2 ()
-  (lexical-let*
-      ((result '())
-       (dfinish (deferred:new
-                  (lambda (x)
-                    (setq result (reverse result))
-                    (or (equal
-                         '("parent get 256" "child get 256") result)
-                        result))))
-       (dfenv-parent (cc:dataflow-environment))
-       (dfenv (cc:dataflow-environment dfenv-parent)))
+  (let* ((result '())
+         (dfinish (deferred:new
+                    (lambda (_)
+                      (setq result (reverse result))
+                      (or (equal
+                           '("parent get 256" "child get 256") result)
+                          result))))
+         (dfenv-parent (cc:dataflow-environment))
+         (dfenv (cc:dataflow-environment dfenv-parent)))
 
     (deferred:$
       (deferred:parallel
@@ -441,9 +432,9 @@
           (deferred:nextc it
             (lambda (x) (push (format "child get %s" x) result))))
         (deferred:nextc (deferred:wait 0.2)
-          (lambda (x) (cc:dataflow-set dfenv-parent "abc" 256))))
+          (lambda (_) (cc:dataflow-set dfenv-parent "abc" 256))))
       (deferred:nextc it
-        (lambda (x) (deferred:callback dfinish))))
+        (lambda (_) (deferred:callback dfinish))))
 
     dfinish))
 
@@ -453,20 +444,20 @@
 ;; Signal
 
 (defun cc:test-signal1 ()
-  (lexical-let*
-      ((result '())
-       (dfinish (deferred:new
-                  (lambda (x)
-                    (setq result (reverse result))
-                    (or (equal
-                          '(
-                            (ls ev1 (1))
-                            (sig ev1 (1))
-                            (ls ev2 (2))
-                            (def ev1 (1))
-                            ) result)
-                        result))))
-       (channel (cc:signal-channel "child")))
+  (let* ((result '())
+         (dfinish (deferred:new
+                    (lambda (_)
+                      (setq result (reverse result))
+                      (or (equal
+                           '(
+                             (ls ev1 (1))
+                             (sig ev1 (1))
+                             (ls ev2 (2))
+                             (def ev1 (1))
+                             )
+                           result)
+                          result))))
+         (channel (cc:signal-channel "child")))
 
     (cc:signal-connect channel 'ev1
                        (lambda (event)
@@ -481,13 +472,13 @@
 
     (deferred:$
       (deferred:next
-        (lambda (x)
+        (lambda (_)
           (cc:signal-send channel 'ev1 1)
           (cc:signal-send channel 'ev2 2)))
       (deferred:nextc it
-        (lambda (x) (deferred:wait 300)))
+        (lambda (_) (deferred:wait 300)))
       (deferred:nextc it
-        (lambda (x)
+        (lambda (_)
           (deferred:callback dfinish))))
 
     dfinish))
@@ -497,35 +488,34 @@
 ;; (cc:debug (cc:test-signal2) "Signal2 : %s" x)
 
 (defun cc:test-signal2 ()
-  (lexical-let*
-      ((result nil)
-       (dfinish (deferred:new
-                  (lambda (x)
-                    (setq result (reverse result))
-                    (or (equal
-                         '(
-                           (pls pev1 (1))
-                           (psig pev1 (1))
-                           (pls ev1 (2))
-                           (ls ev1 (3))
-                           (sig ev1 (3))
-                           (pls ev2 (4))
-                           (pls ev2 (5))
+  (let* ((result nil)
+         (dfinish (deferred:new
+                    (lambda (_)
+                      (setq result (reverse result))
+                      (or (equal
+                           '(
+                             (pls pev1 (1))
+                             (psig pev1 (1))
+                             (pls ev1 (2))
+                             (ls ev1 (3))
+                             (sig ev1 (3))
+                             (pls ev2 (4))
+                             (pls ev2 (5))
 
-                           (ls pev1 (1))
-                           (ls ev1 (2))
+                             (ls pev1 (1))
+                             (ls ev1 (2))
 
-                           (sig ev1 (2))
-                           (def ev1 (3))
-                           (ls ev2 (4))
-                           (ls ev2 (5))
+                             (sig ev1 (2))
+                             (def ev1 (3))
+                             (ls ev2 (4))
+                             (ls ev2 (5))
 
-                           (def ev1 (2))
-                           )
-                         result)
-                        result))))
-       (parent-channel (cc:signal-channel "parent"))
-       (channel (cc:signal-channel "child" parent-channel)))
+                             (def ev1 (2))
+                             )
+                           result)
+                          result))))
+         (parent-channel (cc:signal-channel "parent"))
+         (channel (cc:signal-channel "child" parent-channel)))
 
     (cc:signal-connect parent-channel 'pev1
                        (lambda (event)
@@ -547,16 +537,16 @@
 
     (deferred:$
       (deferred:next
-        (lambda (x)
+        (lambda (_)
           (cc:signal-send parent-channel 'pev1 1)
           (cc:signal-send parent-channel 'ev1  2)
           (cc:signal-send channel        'ev1  3)
           (cc:signal-send parent-channel 'ev2  4)
           (cc:signal-send-global channel 'ev2  5)))
       (deferred:nextc it
-        (lambda (x) (deferred:wait 300)))
+        (lambda (_) (deferred:wait 300)))
       (deferred:nextc it
-        (lambda (x)
+        (lambda (_)
           (deferred:callback-post dfinish))))
 
     dfinish))
@@ -572,40 +562,40 @@
   (setq cc:test-fails 0)
   (deferred:$
     (deferred:parallel
-      (loop for i in '(cc:test-fib-gen
-                       cc:test-thread
-                       cc:test-semaphore1
-                       cc:test-semaphore2
-                       cc:test-dataflow-simple1
-                       cc:test-dataflow-simple2
-                       cc:test-dataflow-simple3
-                       cc:test-dataflow-simple4
-                       cc:test-dataflow-signal
-                       cc:test-dataflow-parent1
-                       cc:test-dataflow-parent2
-                       cc:test-signal1
-                       cc:test-signal2
-                       )
-            collect (cons i (deferred:timeout 5000 "timeout" (funcall i)))))
+      (cl-loop for i in '(cc:test-fib-gen
+                          cc:test-thread
+                          cc:test-semaphore1
+                          cc:test-semaphore2
+                          cc:test-dataflow-simple1
+                          cc:test-dataflow-simple2
+                          cc:test-dataflow-simple3
+                          cc:test-dataflow-simple4
+                          cc:test-dataflow-signal
+                          cc:test-dataflow-parent1
+                          cc:test-dataflow-parent2
+                          cc:test-signal1
+                          cc:test-signal2
+                          )
+               collect (cons i (deferred:timeout 5000 "timeout" (funcall i)))))
     (deferred:nextc it
       (lambda (results)
         (pop-to-buffer
          (with-current-buffer (get-buffer-create "*cc:test*")
            (erase-buffer)
-           (loop for i in results
-                 for name = (car i)
-                 for result = (cdr i)
-                 with fails = 0
-                 do (insert (format "%s : %s\n" name
-                                    (if (eq t result) "OK"
-                                      (format "FAIL > %s" result))))
-                 (unless (eq t result) (incf fails))
-                 finally
-                 (goto-char (point-min))
-                 (insert (format "Test Finished : %s\nTests Fails: %s / %s\n"
-                                 (format-time-string   "%Y/%m/%d %H:%M:%S" (current-time))
-                                 fails (length results)))
-                 (setq cc:test-fails fails))
+           (cl-loop for i in results
+                    for name = (car i)
+                    for result = (cdr i)
+                    with fails = 0
+                    do (insert (format "%s : %s\n" name
+                                       (if (eq t result) "OK"
+                                         (format "FAIL > %s" result))))
+                    (unless (eq t result) (cl-incf fails))
+                    finally
+                    (goto-char (point-min))
+                    (insert (format "Test Finished : %s\nTests Fails: %s / %s\n"
+                                    (format-time-string   "%Y/%m/%d %H:%M:%S" (current-time))
+                                    fails (length results)))
+                    (setq cc:test-fails fails))
            (message (buffer-string))
            (current-buffer)))
         (setq cc:test-finished-flag t))))
